import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:netease_live_fluttify/src/android/android.export.g.dart';
import 'package:netease_live_fluttify/src/facade/models.dart';
import 'package:netease_live_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';
import 'extensions.dart';

class LivePusher with WidgetsBindingObserver {
  /// 创建推流
  static Future<LivePusher> create() async {
    final pusher = LivePusher._();
    await platform(
      android: (pool) async {
        final context = await android_app_Application.get();
        final param =
            await com_netease_LSMediaCapture_lsMediaCapture_LsMediaCapturePara
                .create__();
        await param.set_Context(context);
        await param
            .setLogLevel(com_netease_LSMediaCapture_lsLogUtil_LogLevel.INFO);
        pusher._androidPusher = await com_netease_LSMediaCapture_lsMediaCapture
            .create__com_netease_LSMediaCapture_lsMediaCapture_LsMediaCapturePara(
                param);
      },
      ios: (pool) async {
        pusher._iosPusher = await LSMediaCapture.create__();
      },
    );
    return pusher;
  }

  com_netease_LSMediaCapture_lsMediaCapture _androidPusher;
  LSMediaCapture _iosPusher;

  bool _isLiving = false;

  LivePusher._() {
    WidgetsBinding.instance.addObserver(this);
  }

  /// 初始化推流
  ///
  /// [pushUrl] 推流地址
  /// [config] 音视频配置
  Future<void> initLiveStream(
    String pushUrl, {
    LiveStreamConfig config = const LiveStreamConfig(),
  }) async {
    return platform(
      android: (pool) async {
        await _androidPusher.initLiveStream(
          await config.androidModel(),
          pushUrl,
        );
      },
      ios: (pool) async {
        await _iosPusher.initLiveStream(pushUrl);
      },
    );
  }

  /// 开始预览
  ///
  /// [preview] SDK提供的预览显示View，用户可以布局到自己的APP中
  /// [frontCamera] 是否默认前置摄像头
  /// [filter] 是否使用SDK内置滤镜
  /// [videoQuality] MEDIUM：标清 480360，HIGH：高清 640480，SUPER：超清 960540，SUPER_HIGH：超高清 1280720
  /// [scale_16x9] 是否按16:9宽高比例
  Future<void> startVideoPreview(
    dynamic preview, {
    bool frontCamera = true,
    bool filter = true,
    VideoQuality videoQuality = VideoQuality.HIGH,
    bool scale_16x9 = false,
  }) async {
    assert(
      preview is com_netease_vcloud_video_render_NeteaseView ||
          preview is UIView,
    );
    return platform(
      android: (pool) async {
        com_netease_LSMediaCapture_lsMediaCapture_VideoQuality _quality;
        switch (videoQuality) {
          case VideoQuality.HIGH:
            _quality =
                com_netease_LSMediaCapture_lsMediaCapture_VideoQuality.HIGH;
            break;
          case VideoQuality.MEDIUM:
            _quality =
                com_netease_LSMediaCapture_lsMediaCapture_VideoQuality.MEDIUM;
            break;
          case VideoQuality.SUPER:
            _quality =
                com_netease_LSMediaCapture_lsMediaCapture_VideoQuality.SUPER;
            break;
          case VideoQuality.SUPER_HIGH:
            _quality = com_netease_LSMediaCapture_lsMediaCapture_VideoQuality
                .SUPER_HIGH;
            break;
          case VideoQuality.HD1080P:
            _quality =
                com_netease_LSMediaCapture_lsMediaCapture_VideoQuality.HD1080P;
            break;
        }
        await _androidPusher.startVideoPreview(
          preview,
          frontCamera,
          filter,
          _quality,
          scale_16x9,
        );
      },
      ios: (pool) async {
        await _iosPusher.startVideoPreview(preview);
      },
    );
  }

  /// 开始推流
  Future<void> startLiveStream() async {
    _isLiving = true;
    return platform(
      android: (pool) async {
        await _androidPusher.startLiveStreaming();
      },
      ios: (pool) async {
        await _iosPusher.startLiveStream((error) {
          print('startLiveStream: ' + error.toString());
        });
      },
    );
  }

  /// 恢复推流
  Future<void> resumeLiveStream() async {
    _isLiving = true;
    return platform(
      android: (pool) {
        return _androidPusher.resumeVideoLiveStream();
      },
      ios: (pool) {
        return _iosPusher.resumeVideoLiveStream();
      },
    );
  }

  /// 停止推流
  Future<void> stopLiveStreaming() async {
    _isLiving = false;
    return platform(
      android: (pool) async {
        await _androidPusher.stopLiveStreaming();
      },
      ios: (pool) async {
        await _iosPusher.stopLiveStream((error) {});
      },
    );
  }

  /// 停止预览
  Future<void> stopVideoPreview() async {
    _isLiving = false;
    return platform(
      android: (pool) async {
        await _androidPusher.stopVideoPreview();
      },
      ios: (pool) async {
        await _iosPusher.pauseVideoPreview();
      },
    );
  }

  /// 切换摄像头
  Future<void> switchCamera() async {
    return platform(
      android: (pool) async {
        await _androidPusher.switchCamera();
      },
      ios: (pool) async {
        await _iosPusher.switchCamera(() {});
      },
    );
  }

  /// 设置滤镜
  ///
  /// 由于Android和iOS两端支持的滤镜不一致, 所以开放两个参数单独设置. 枚举具体含义请参考枚举
  /// 类的注释.
  Future<void> setFilterType({
    @required FilterTypeAndroid filterTypeAndroid,
    @required FilterTypeIOS filterTypeIOS,
  }) async {
    return platform(
      android: (pool) async {
        await _androidPusher.setFilterType(filterTypeAndroid.androidModel());
      },
      ios: (pool) async {
        await _iosPusher.setFilterType(filterTypeIOS.iosModel());
      },
    );
  }

  /// 设置滤镜强度
  Future<void> setFilterStrength(double strength) async {
    assert(strength >= 0 && strength <= 1, '滤镜强度范围为0 - 1!');
    return platform(
      android: (pool) async {
        await _androidPusher.setFilterStrength(strength);
      },
      ios: (pool) async {
        await _iosPusher.setWhiteningFilterIntensity(strength);
      },
    );
  }

  /// 设置磨皮强度
  ///
  /// [strength] 强度 0 - 1
  Future<void> setSmoothLevel(double strength) async {
    assert(strength >= 0 && strength <= 1, '磨皮强度范围为0 - 1!');
    return platform(
      android: (pool) async {
        await _androidPusher.setBeautyLevel((strength * 5).round());
      },
      ios: (pool) async {
        await _iosPusher.setSmoothFilterIntensity(strength.toDouble());
      },
    );
  }

  /// 打开/关闭 闪光灯
  Future<void> flash(bool on) async {
    return platform(
      android: (pool) async {
        await _androidPusher.setCameraFlashPara(on);
      },
      ios: (pool) async {
        await _iosPusher.set_flash(on);
      },
    );
  }

  /// 摄像头缩放
  Future<void> zoom(int level) async {
    return platform(
      android: (pool) async {
        await _androidPusher.setCameraZoomPara(level);
      },
      ios: (pool) async {
        await _iosPusher.set_zoomScale(level.toDouble());
      },
    );
  }

  /// 调节曝光度
  Future<void> setExposureCompensation(int exposure) async {
    return platform(
      android: (pool) async {
        await _androidPusher.setExposureCompensation(exposure);
      },
      ios: (pool) async {
        await _iosPusher.adjustExposure(exposure.toDouble());
      },
    );
  }

  /// 销毁资源
  Future<void> dispose() async {
    WidgetsBinding.instance.removeObserver(this);
    return platform(
      android: (pool) async {
        await _androidPusher.destroyVideoPreview();
        await _androidPusher.uninitLsMediaCapture(false);
      },
      ios: (pool) async {
        await _iosPusher.unInitLiveStream();
      },
    );
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) async {
    if (Platform.isAndroid) {
      switch (state) {
        case AppLifecycleState.resumed:
          if (_isLiving) {
            await _androidPusher.resumeVideoEncode();
            await _androidPusher.resumeAudioEncode();
          }
          break;
        case AppLifecycleState.paused:
          if (_isLiving) {
            await _androidPusher.backgroundVideoEncode();
            await _androidPusher.backgroundAudioEncode();
          }
          break;
        default:
          break;
      }
    }
  }
}
